from direct.gui.DirectGui import *
from direct.task import Task
from pandac.PandaModules import *
import math

HUD = 'data/gui/hud/'
base.setFrameRateMeter(True)
class Hud:
    
    def __init__(self,ship,world):
        self.ship = ship
        self.lastView = self.ship.pilot.view
        #self.ship.node.hide()
        self.world = world
        self.trackerNodePath = {}
        self.trackerModel = loader.loadModel(HUD+'enemy-arrow.egg')
        self.dotModel = loader.loadModel(HUD+'enemy-dot.egg')
        self.crosshair = loader.loadModel(HUD+'crosshair.egg')
        self.crosshair.reparentTo(aspect2d)
        self.crosshair.setScale(0.1)
        self.iris = loader.loadModel(HUD+'iris.egg')
        self.iris.reparentTo(aspect2d)
        if self.lastView == '3rdperson':self.crosshair.hide();self.iris.setScale(1.6)
        
        self.ammoHUD = DirectLabel(text = 'Ammo: '+str(self.ship.ammo),
                    relief = None,pos = (base.a2dLeft + 0.1, 0, -0.8),scale = 0.04)
        self.ammoHUD.setColor(255,255,255)
        self.armorHUD = DirectLabel(text = 'Armor: '+str(self.ship.armor),
                    relief = None,pos = (base.a2dLeft + 0.5, 0, -0.8),scale = 0.04)
        self.armorHUD.setColor(255,255,255)
        self.shieldHUD = DirectLabel(text = 'Shield: '+str(self.ship.shield),
                    relief = None,pos = (base.a2dLeft + 0.9, 0, -0.8),scale = 0.04)
        self.shieldHUD.setColor(255,255,255)
        self.structureHUD = DirectLabel(text = 'Structure: '+str(self.ship.structure),
                    relief = None,pos = (base.a2dLeft + 1.3, 0, -0.8),scale = 0.04)
        self.structureHUD.setColor(255,255,255)
        self.weaponHUD = DirectLabel(text = 'Weapon: '+str(self.ship.weapon),
                    relief = None,pos = (base.a2dLeft + 1.7, 0, -0.8),scale = 0.04)
        self.weaponHUD.setColor(255,255,255)
        self.speedHUD = DirectLabel(text = 'Speed: '+str(self.ship.speed),
                    relief = None,pos = (base.a2dLeft + 2.1, 0, -0.8),scale = 0.04)
        self.speedHUD.setColor(255,255,255)
        self.fuelHUD = DirectLabel(text = 'Fuel: '+str(self.ship.armor),
                    relief = None,pos = (base.a2dLeft + 0.1, 0, -0.9),scale = 0.04)
        self.fuelHUD.setColor(255,255,255)
        self.fragHUD = DirectLabel(text = 'Frags: '+str(self.ship.armor),
                    relief = None,pos = (base.a2dLeft + 0.5, 0, -0.9),scale = 0.04)
        self.fragHUD.setColor(255,255,255)
        
    def get2D(self, nodePath):
        
        p3d = base.cam.getRelativePoint(nodePath, Point3(0,0,0))
        p2d = Point2()
        
        
        if base.cam.node().getLens().project(p3d, p2d):
            return p2d
        return None
        
    def update(self,task):
        if self.ship.pilot.view == '3rdperson' and self.lastView == '1stperson': 
            self.crosshair.hide()
            self.lastView = '3rdperson'
            
            self.iris.setScale(1.6)
            
        if self.ship.pilot.view == '1stperson' and self.lastView == '3rdperson': 
            self.crosshair.show()
            self.lastView = '1stperson'
            
            self.iris.setScale(1)
            
        self.ammoHUD['text'] = 'Ammo: '+str(self.ship.ammo)
        self.armorHUD['text'] = 'Armor: '+str(self.ship.armor)
        self.shieldHUD['text'] = 'Shield: '+str(self.ship.shield)
        self.structureHUD['text'] = 'Structure: '+str(self.ship.structure)
        self.weaponHUD['text'] = 'Weapon: '+str(self.ship.weapon)
        self.fuelHUD['text'] = 'Fuel: '+str(self.ship.fuel)
        self.fragHUD['text'] = 'Frags: '+str(self.ship.frags)
        self.speedHUD['text'] = 'Speed: '+str(self.ship.speed)
        
        if len(self.trackerNodePath) > len(self.world.ships):
            for key in self.trackerNodePath:
                self.trackerNodePath[key].detachNode()
            del self.trackerNodePath
            self.trackerNodePath = {}
        
        for key in self.world.ships:
            relativeVec = self.world.ships[key].node.getPos(self.ship.node)
            if relativeVec.getY() < 0:relativeVec.setY(0)
            relativeVec.normalize()
            if self.lastView == '1stperson':
                relativeVec=relativeVec/2
            else:
                relativeVec=relativeVec*0.8
            
            
            if self.trackerNodePath.has_key(key):
                self.trackerNodePath[key].setPos(relativeVec)
                pos2d = self.get2D(self.world.ships[key].node)
                if pos2d:
                    length = math.sqrt(pos2d.getX()*pos2d.getX()+pos2d.getY()*pos2d.getY())
                    brightness = length
                    if brightness > 1:brightness = 1
                    if brightness < 0:brightness = 0
                    self.trackerNodePath[key].find('**/*arrow*').hide()
                    self.trackerNodePath[key].find('**/*dot*').show()
                    self.trackerNodePath[key].find('**/*dot*').setColor(1,1,1,brightness)
                else:
                    vec1 = Vec3(self.trackerNodePath[key].find('**/*arrow*').getPos(aspect2d))
                    vec2 = Vec3(0,0,0.5)
                    vec1.normalize()
                    vec2.normalize()
                    newR = vec1.angleDeg(vec2)
                    if vec1.getX() < 0.0: newR = newR*(-1)
                    self.trackerNodePath[key].find('**/*arrow*').show()
                    if self.world.ships[key].node == self.ship.node: self.trackerNodePath[key].find('**/*arrow*').hide()
                    self.trackerNodePath[key].find('**/*arrow*').setR(newR)
                    self.trackerNodePath[key].find('**/*dot*').hide()
            else:
                self.trackerNodePath[key] = aspect2d.attachNewNode(str(key))
                dot = self.trackerNodePath[key].attachNewNode('enemy-dot')
                arrow = self.trackerNodePath[key].attachNewNode('enemy-arrow')
                self.trackerModel.instanceTo(arrow)
                self.dotModel.instanceTo(dot)
                self.trackerNodePath[key].setPos(relativeVec)
                self.trackerNodePath[key].setScale(0.05)
            
        return Task.cont
        