import direct.directbase.DirectStart
from direct.showbase import Audio3DManager
import math, sys, random, os
from pandac.PandaModules import *
from direct.showbase import DirectObject
from direct.task import Task

SOUNDS = 'data/sounds/'
sndFire = SOUNDS+'fire1.ogg'
sndHit = SOUNDS+'hit1.ogg'
sndWorldHit = SOUNDS+'hit2.ogg'
sndEngine = SOUNDS+'engine1.ogg'
sndExplosion = SOUNDS+'expl1.ogg'

class SoundFx:
    def __init__(self):
        base.enableMusic(1)
        base.enableSoundEffects(1)
        base.enableAllAudio() 
        self.audio3d = Audio3DManager.Audio3DManager(base.sfxManagerList[0], camera)
        self.audio3d.setDropOffFactor(0.01)
        self.audio3d.setListenerVelocityAuto()
        self.sounds = {}
        self.worldHitNodes = {} 
        self.explosionNodes = {}
        
    def addSoundsToShip(self, ship):
        id = str(ship.id)
        nodePath = ship.node
        self.worldHitNodes[id] = render.attachNewNode(id+'worldHitNode')
        self.explosionNodes[id] = render.attachNewNode(id+'explosionNode')
        self.sounds[id+'-exp'] = self.audio3d.loadSfx(sndExplosion)
        self.sounds[id+'-wHit'] = self.audio3d.loadSfx(sndWorldHit)
        self.sounds[id+'-fire'] = self.audio3d.loadSfx(sndFire)
        self.sounds[id+'-hit'] = self.audio3d.loadSfx(sndHit)
        self.sounds[id+'-eng'] = self.audio3d.loadSfx(sndEngine)
        self.audio3d.setSoundVelocityAuto(self.sounds[id+'-eng'])
        self.audio3d.setSoundVelocityAuto(self.sounds[id+'-wHit'])
        self.audio3d.setSoundVelocityAuto(self.sounds[id+'-exp'])
        self.audio3d.setSoundVelocityAuto(self.sounds[id+'-hit'])
        self.audio3d.setSoundVelocityAuto(self.sounds[id+'-fire'])
        self.audio3d.attachSoundToObject(self.sounds[id+'-wHit'], self.worldHitNodes[id])
        self.audio3d.attachSoundToObject(self.sounds[id+'-exp'], self.explosionNodes[id])
        self.audio3d.attachSoundToObject(self.sounds[id+'-fire'], nodePath)
        self.audio3d.attachSoundToObject(self.sounds[id+'-hit'], nodePath)
        self.audio3d.attachSoundToObject(self.sounds[id+'-eng'], nodePath)
        
    def removeSoundsFromShip(self, ship):
        id = str(ship.id)
        self.audio3d.detachSound( self.sounds[id+'-exp'])
        self.audio3d.detachSound( self.sounds[id+'-wHit'])
        self.audio3d.detachSound(self.sounds[id+'-fire'])
        self.audio3d.detachSound(self.sounds[id+'-eng'])
        self.audio3d.detachSound(self.sounds[id+'-hit'])
        del self.sounds[id+'-exp']
        del self.sounds[id+'-wHit']
        del self.sounds[id+'-fire']
        del self.sounds[id+'-eng']
        del self.sounds[id+'-hit']
        del self.worldHitNodes[id]
        del self.explosionNodes[id]
        
    def playExplosion(self, ship, volume=0.5):
        id = str(ship.id)
        pos = ship.node.getPos()
        self.explosionNodes[id].setPos(pos)
        self.sounds[id+'-exp'].setVolume(volume)
        self.sounds[id+'-exp'].play()
        
    def playWorldHit(self, ship, pos=Vec3(0,0,0), volume=0.2):
        id = str(ship.id)
        self.worldHitNodes[id].setPos(pos)
        self.sounds[id+'-wHit'].setVolume(volume)
        self.sounds[id+'-wHit'].play()
        
    def playEngine(self, ship, volume=0.2):
        id = str(ship.id)
        self.sounds[id+'-eng'].setVolume(volume)
        self.sounds[id+'-eng'].setLoop(True)
        self.sounds[id+'-eng'].play()
        
    def stopEngine(self, ship, volume=0.0):
        id = str(ship.id)
        self.sounds[id+'-eng'].setVolume(volume)
        self.sounds[id+'-eng'].setLoop(False)
        self.sounds[id+'-eng'].stop()
        
    def playHit(self, ship, volume=0.5):
        id = str(ship.id)
        self.sounds[id+'-hit'].setVolume(volume)
        self.sounds[id+'-hit'].play()
        
    def stopHit(self, ship, volume=0.0):
        id = str(ship.id)
        self.sounds[id+'-hit'].setVolume(volume)
        self.sounds[id+'-hit'].stop()
        
    def playFire(self, ship, volume=0.2):
        id = str(ship.id)
        self.sounds[id+'-fire'].setVolume(volume)
        self.sounds[id+'-fire'].play()
        
    def stopFire(self, ship, volume=0.0):
        id = str(ship.id)
        self.sounds[id+'-fire'].setVolume(volume)
        self.sounds[id+'-fire'].stop()